import { useEditor } from "@/modules/editor";
import { Alignment } from "@ckeditor/ckeditor5-alignment";
import { Bold, Italic } from "@ckeditor/ckeditor5-basic-styles";
import { InlineEditor } from "@ckeditor/ckeditor5-editor-inline";
import { Essentials } from "@ckeditor/ckeditor5-essentials";
import { FontColor, FontFamily, FontSize } from "@ckeditor/ckeditor5-font";
import { Link } from "@ckeditor/ckeditor5-link";
import { Paragraph } from "@ckeditor/ckeditor5-paragraph";
import { css } from "@linaria/core";
import { defineComponent, onUnmounted, watch, watchEffect , ref} from "vue";
import { string} from "vue-types";
import { useCompData } from ".";
import { View } from "../View";
import { nextTick } from "process";

export const Component = defineComponent({
  props: {
    compId: string().def(""),
  },
  setup(props) {
    const comp = useCompData(props.compId);
    const { store, actions , helper, controls} = useEditor();

    const fontSizeOptions=[];
    const list = [12, 14, 16, 18, 20, 24, 28, 32, 38, 42, 46, 52, 60]
    for (const s of list) {
      fontSizeOptions.push({title: s+"", model: s + "px"})
    }
    const config = {
      language: "zh-cn",
      plugins: [
        Essentials,
        Bold,
        Italic,
        Link,
        Paragraph,
        FontColor,
        FontSize,
        FontFamily,
        Alignment,
      ],
      fontSize: {
        options: fontSizeOptions,
        supportAllValues:true,
      },
      toolbar: {
        items: [
          // "undo",
          // "redo",
          // "|",
          "fontColor",
          "fontsize",
          "bold",
          "italic",
          "|",
          "alignment",
          // "|",
          // "link",
        ],
      },
    };

    let editorInstance: InlineEditor;

    watch(
      () => comp.value,
      () => {
        if (!store.textEditingState) {
          editorInstance?.setData(comp.value);
        }
      }
    );

    let blurCanceler: any = null;
    watchEffect(() => {
      blurCanceler?.();
      if (store.currCompId === props.compId) {
        blurCanceler = blurHandle();
      }
    });

    onUnmounted(() => {
      blurCanceler?.();
    });

    function blurHandle() {
      function blur(e: MouseEvent) {
        const target = e.target as HTMLElement;
        if (
          editorInstance.ui.view.toolbar.element?.contains(target) ||
          editorInstance.ui.view.editable.element?.contains(target)
        ) {
          return;
        }
        store.setTextEditingState(false);
        actions.submitUpdate();
      }
      document.addEventListener("mousedown", blur, {
        capture: true,
      });
      return () => {
        document.removeEventListener("mousedown", blur, { capture: true });
      };
    }
    const inputRef = ref<any>();
    if (store.isEditMode) {
      actions.on("textFocus", (compId, focus:boolean)=>{
        if (compId != props.compId) return;

        if (focus) {
          editorInstance.focus();
          return;
        } 
      })  
    }
    
    return () => (
      <View
        class={[textStyle, store.currCompId === props.compId && "drag-disable"]}
        compId={props.compId}
      >
        <ckeditor
          class={textStyle}
          ref={inputRef}
          editor={InlineEditor}
          onFocus={() => {
            console.log("on  onn focus !!!")
            store.setTextEditingState(true);
          }}
          onInput={(value: any) => {
            if (editorInstance && comp.value !== value) {
              // actions.updateCompData(comp, "value", value);
              nextTick(()=>{
                actions.updateCompDatas(comp, ["value", "layout.size.1"], [value, helper.pxToDesignSize(inputRef.value?.$el.clientHeight)])
                controls.selectCtrl.upgateGizmoStyle();
              })
            }
          }}
          onReady={(editor: InlineEditor) => {
            editorInstance = editor;
            editor.setData(comp.value);
            if (store.isPreview) {
              editor.enableReadOnlyMode("editor");
            }
          }}
          config={config}
        />
      </View>
    );
  },
});

const textStyle = css`
  font-size: 0.24rem;
  width: 100%;
  color: #666;
  p {
    margin: 0;
  }
  .ck.ck-editor__editable_inline {
    cursor: text;
    overflow: hidden;
    pointer-events: none;

    > :last-child,
    > :first-child {
      margin-top: 0;
      margin-bottom: 0;
    }
  }

  &.drag-disable {
    .ck.ck-editor__editable_inline {
      pointer-events: auto;
    }
  }
`;
